@@ -21,13 +21,19 @@
 /*! \file
  *
  * \brief Custom Comma Separated Value CDR records.
- * 
+ *
+ * \author Mark Spencer <markster@digium.com>
+ *
  * \arg See also \ref AstCDR
  *
  * Logs in LOG_DIR/cdr_custom
  * \ingroup cdr_drivers
  */
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 70841 $")
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
@@ -37,10 +43,6 @@
 #include <unistd.h>
 #include <time.h>
 
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 70804 $")
-
 #include "asterisk/channel.h"
 #include "asterisk/cdr.h"
 #include "asterisk/module.h"
@@ -55,13 +57,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 70804 $")
 
 AST_MUTEX_DEFINE_STATIC(lock);
 
-static char *desc = "Customizable Comma Separated Values CDR Backend";
-
 static char *name = "cdr-custom";
 
 static FILE *mf = NULL;
 
-static char master[AST_CONFIG_MAX_PATH];
+static char master[PATH_MAX];
 static char format[1024]="";
 
 static int load_config(int reload) 
@@ -77,9 +77,9 @@ static int load_config(int reload)
 		var = ast_variable_browse(cfg, "mappings");
 		while(var) {
 			if (!ast_strlen_zero(var->name) && !ast_strlen_zero(var->value)) {
-				if (strlen(var->value) > (sizeof(format) - 2))
+				if (strlen(var->value) > (sizeof(format) - 1))
 					ast_log(LOG_WARNING, "Format string too long, will be truncated, at line %d\n", var->lineno);
-				strncpy(format, var->value, sizeof(format) - 2);
+				ast_copy_string(format, var->value, sizeof(format) - 1);
 				strcat(format,"\n");
 				snprintf(master, sizeof(master),"%s/%s/%s", ast_config_AST_LOG_DIR, name, var->name);
 				if (var->next) {
@@ -137,12 +137,7 @@ static int custom_log(struct ast_cdr *cdr)
 	return 0;
 }
 
-char *description(void)
-{
-	return desc;
-}
-
-int unload_module(void)
+static int unload_module(void)
 {
 	if (mf)
 		fclose(mf);
@@ -150,31 +145,29 @@ int unload_module(void)
 	return 0;
 }
 
-int load_module(void)
+static int load_module(void)
 {
 	int res = 0;
 
 	if (!load_config(0)) {
-		res = ast_cdr_register(name, desc, custom_log);
+		res = ast_cdr_register(name, ast_module_info->description, custom_log);
 		if (res)
 			ast_log(LOG_ERROR, "Unable to register custom CDR handling\n");
 		if (mf)
 			fclose(mf);
-	}
-	return res;
+		return res;
+	} else 
+		return AST_MODULE_LOAD_DECLINE;
 }
 
-int reload(void)
+static int reload(void)
 {
 	return load_config(1);
 }
 
-int usecount(void)
-{
-	return 0;
-}
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Customizable Comma Separated Values CDR Backend",
+		.load = load_module,
+		.unload = unload_module,
+		.reload = reload,
+	       );
 
-char *key()
-{
-	return ASTERISK_GPL_KEY;
-}
