@@ -44,7 +44,7 @@ extern "C" {
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 79174 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 83432 $")
 
 #ifdef __cplusplus
 }
@@ -303,9 +303,9 @@ static void oh323_destroy_peer(struct oh323_peer *peer)
 	free(peer);
 }
 
-static int oh323_simulate_dtmf_end(void *data)
+static int oh323_simulate_dtmf_end(const void *data)
 {
-	struct oh323_pvt *pvt = data;
+	struct oh323_pvt *pvt = (struct oh323_pvt *)data;
 
 	if (pvt) {
 		ast_mutex_lock(&pvt->lock);
