@@ -34,7 +34,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 48513 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 83177 $")
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -145,6 +145,8 @@ static int acf_curl_exec(struct ast_channel *chan, char *cmd, char *info, char *
 
 	AST_STANDARD_APP_ARGS(args, info);	
 
+	ast_autoservice_start(chan);
+
 	if (!curl_internal(&chunk, args.url, args.postdata)) {
 		if (chunk.memory) {
 			chunk.memory[chunk.size] = '\0';
@@ -158,6 +160,8 @@ static int acf_curl_exec(struct ast_channel *chan, char *cmd, char *info, char *
 		ast_log(LOG_ERROR, "Cannot allocate curl structure\n");
 	}
 
+	ast_autoservice_stop(chan);
+
 	ast_module_user_remove(u);
 
 	return 0;
