@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 76891 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 83773 $")
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -782,7 +782,7 @@ int ast_app_group_split_group(const char *data, char *group, int group_max, char
 	if (!ast_strlen_zero(grp))
 		ast_copy_string(group, grp, group_max);
 	else
-		res = -1;
+		*group = '\0';
 
 	if (!ast_strlen_zero(cat))
 		ast_copy_string(category, cat, category_max);
@@ -814,8 +814,10 @@ int ast_app_group_set_channel(struct ast_channel *chan, const char *data)
 		}
 	}
 	AST_LIST_TRAVERSE_SAFE_END
-	
-	if ((gi = calloc(1, len))) {
+
+	if (ast_strlen_zero(group)) {
+		/* Enable unsetting the group */
+	} else if ((gi = calloc(1, len))) {
 		gi->chan = chan;
 		gi->group = (char *) gi + sizeof(*gi);
 		strcpy(gi->group, group);
